from mag_tools.model.base_enum import BaseEnum


class MenuType(BaseEnum):
    DROP_DOWN = ("drop_down", "下拉菜单")
    POPUP = ("popup", "弹出菜单")
    CONTEXT = ("context", "上下文菜单")
    MENU_ITEM = ("menu_item", "菜单项")


from typing import Optional

from mag_tools.utils.common.string_utils import StringUtils

from mag_test.model.action_type import ActionType
from mag_test.model.control_type import ControlType
from mag_test.model.menu_type import MenuType
from mag_test.model.init_status import InitStatus


class Control:
    def __init__(self, name: Optional[str] = None, control_type: Optional[ControlType] = None, automation_id: Optional[str] = None,
                 action: Optional[ActionType] = None, class_name: Optional[str] = None,
                 menu_type: Optional[MenuType] = None, init_status: Optional[InitStatus] = None, is_composite: bool = False):
        self.name = name
        self.control_type = control_type
        self.automation_id = automation_id
        self.action = action
        self.menu_type = menu_type
        self.class_name = class_name
        self.init_status = init_status
        self.is_composite = is_composite

    @classmethod
    def parse_name_id_path(cls, name_id_path: str, control_type: Optional[ControlType] = None):
        name, automation_id, action, init_status = None, None, None, None
        if name_id_path:
            name_id, action_name, _ = StringUtils.split_by_keyword(name_id_path, '{}')
            name = name_id if name_id and not name_id.isdigit() else None
            automation_id = name_id if name_id and name_id.isdigit() else None

            if control_type in {ControlType.TREE, ControlType.TABLE}:
                init_status = InitStatus.of_code(action_name)
            else:
                action = ActionType.of_code(action_name) if action_name else ActionType.default_action(control_type)

        return name, automation_id, action, init_status

    @classmethod
    def parse_id_path(cls, id_path: str, control_type: Optional[ControlType] = None):
        automation_id, action, init_status = None, None, None

        if id_path:
            automation_id, action_name, _ = StringUtils.split_by_keyword(id_path, '{}')
            if control_type in {ControlType.TREE, ControlType.TABLE}:
                init_status = InitStatus.of_code(action_name)
            else:
                action = ActionType.of_code(action_name) if action_name else ActionType.default_action(control_type)

        return automation_id, action, init_status

    def __str__(self) -> str:
        attributes = {k: v for k, v in self.__dict__.items() if v is not None}
        return f"Control({', '.join(f'{k}={v}' for k, v in attributes.items())})"


import json
import os

import allure
import pytest
from mag_tools.utils.common.time_probe import TimeProbe

from mag_test.bean.base_test import BaseTest
from mag_test.bean.step import Step
from mag_test.core.app_driver import AppDriver
from mag_test.model.control_type import ControlType
from mag_test.model.test_component_type import TestComponentType
from mag_test.model.usage_status import UsageStatus


class Function(BaseTest):
    def __init__(self, home_dir:str, plan_id:str, function_id:str, index:int, name:str, status:UsageStatus=UsageStatus.NORMAL):
        super().__init__(home_dir, name, index, TestComponentType.FUNCTION, None, status)
        self.__plan_id = plan_id
        self.__id = function_id  # 功能标识
        self.__steps = []

        self.__read()

    @pytest.mark.benchmark
    def start(self, driver:AppDriver, probe: TimeProbe)->AppDriver:
        """
        启动测试功能
        :param probe: 时间探针
        :param driver: AppDriver
        """
        if self._status == UsageStatus.NORMAL:
            with allure.step(f"{self._index} {self._name}"):  # 描述测试功能
                test_failed = False
                for step in self.__steps:
                    if test_failed:
                        step.skip()
                    else:
                        driver = step.start(driver, probe)
                        if step.is_fail():
                            self.fail('该功能测试失败')
                            test_failed = True
        return driver

    def __read(self):
        function_file = os.path.join(self.script_dir, self.__plan_id, f'{self.__id}.json')
        with open(function_file, 'r', encoding='utf-8') as file:
            data = json.load(file)
            if data['name']:
                self._name = data['name']
                self._status = UsageStatus[data['status']]

            for index, item in enumerate(data.get('steps', []), start=1):
                step = Step(home_dir=self._home_dir,
                            name=item.get('step_name', None),
                            control_name=item.get('control_name', None),
                            control_type=ControlType[item.get('control_type')] if item.get('control_type') else None,
                            automation_id=item.get('id', None),
                            value=item.get('value', None),
                            function_index=self._index,
                            step_index=index,
                            parent_name=item.get('parent', None),
                            parent_type=ControlType[item.get('parent_type')] if item.get('parent_type') else None,
                            pop_window=item.get('pop', None),
                            status=UsageStatus[item.get('status', None)])
                self.__steps.append(step)